/*global define*/
define([
    'sandbox/dom',
    'sandbox/event',
    'sandbox/request',
    'backbone.marionette',
    '../constants/device',
    '../templates/views/editor/editor',
    '../templates/includes/button',
    '../views/outline',
    'handlebars',
    'underscore',
    'app/views/mouse'
], function ($, vent, request, Marionette, device, template, button_tmpl, Outline, Handlebars, _, Mouse) {

    'use strict';

    var mouse = new Mouse(),
        sitecanvasDeviceClassNames = _.map(device, function (deviceString) {
            return 'site-' + deviceString;
        }).join(' ');


    return Marionette.Layout.extend({

        pageModel: null,

        attributes: {
            'data-view': 'editor'
        },

        selectedGrid: null,

        toolbar: null,

        rows: null,

        rowViews: [],

        template: template,

        className: 'site-preview',

        regions: {
            'tools': '#tools'
        },

        ui: {
            iframe: 'iframe',
            'content': '.content',
            'sitecanvas': '.site-canvas'
        },

        initialize: function () {
            this.app = this.options.app;

            Handlebars.registerPartial('includes/button', button_tmpl);

            this.listenTo(vent, 'editor.view:show', this.openPage);
            this.listenTo(vent, 'editor.view:close', this.closePage);
            this.listenTo(vent, 'editor.selected', this.selectGrid);
            this.listenTo(vent, 'editor.content:change', this.changeContent);
            this.listenTo(vent, 'editor.content:add', this.addContent);
            this.listenTo(vent, 'editor.content:remove', this.removeContent);
            this.listenTo(vent, 'device', this.updateSiteCanvasClass);
            this.listenTo(vent, 'editor.row:add', this.addRow);

            this.listenTo(vent, 'hovered.last', this.setLastHovered);
        },

        // We'll remember this internally.
        setLastHovered: function (element) {
            this._$lastHovered = $(element);
        },

        onRender: function () {
            this.tools.show(new Outline({
                iframe: this.ui.iframe
            }));
        },

        updateSiteCanvasClass: function (deviceString) {
            this.$el
                .removeClass(sitecanvasDeviceClassNames)
                .addClass('site-' + deviceString);
        },

        openPage: function (pageModel) {
            var that = this;
            this.pageModel = pageModel;

            this.ui.iframe.one('load', function () {
                mouse.setElement($(this));
                mouse.options.outline = that.tools.currentView;
            });
            this.ui.iframe.attr('src', pageModel.url());
        },

        closePage: function (pageModel) {
            if (this.pageModel && this.pageModel.id === pageModel.id) {
                this.pageModel = null;
                this.ui.iframe.attr('src', '');
                mouse.remove();
            }
        },

        addRow: function (row) {
            this.pageModel.get('rows').add(row);
            this.savePage();
        },

        savePage: function () {
            var onDoneCallback = function () {
                this.openPage(this.pageModel);
            };
            this.pageModel.save().done(_.bind(onDoneCallback, this));
        },

        getLastHovered : function () {
            return this._$lastHovered;
        },

        addContent: function (data) {
            this.sendContent(this.getLastHovered(), data);
        },

        // TODO
        // This will have to go elsewhere.
        // For the moment this is only enabling editing
        // of text elements and image sources. Should be delegated
        // to some other intelligence,
        // with knowledge about the models of the rendered
        // storefront.
        changeContent: function (e) {
            var $target = $(e.target),
                text, src;

            if ($target.is('[data-type="text"][data-view="content"]')) {
                /*global prompt*/
                text = prompt('Change text', $target.text());
                if (text) {
                    this.sendContent($target.parent(), {
                        text: text,
                        objectType: '.Text'
                    });
                }
            } else {
                if ($target.is('[data-type="image"][data-view="content"]')) {
                    /*global prompt*/
                    src = prompt('Change image source', $target.attr('src'));
                    if (src) {
                        this.sendContent($target.parent(), {
                            src: src,
                            title: 'Renamed image',
                            objectType: '.Image'
                        });
                    }
                }
            }

        },

        sendContent: function (target, data) {
            var self = this,
                pageId = this.pageModel.id,
                columnId = target.data('id');
            if (target.is('div[data-view="column"]')) {
                request({
                    dataType: 'html',
                    contentType: 'application/json',
                    type: 'post',
                    url: 'page/' + pageId + '/column/' + columnId + '/content',
                    data: JSON.stringify(data)
                }).done(function (response) {
                    target.empty().append(response);
                    self.pageModel.fetch();
                });
            } else {
                this.pageModel.addContent(data);
                this.savePage();
            }
        },

        removeContent: function () {
            /*global confirm*/
            var self = this,
                target = this.getLastHovered().parent(),
                pageId = this.pageModel.id,
                columnId = target.data('id'),
                destroyModel = function () {
                    request({
                        contentType: 'application/json',
                        type: 'delete',
                        url: 'page/' + pageId + '/column/' + columnId + '/content'
                    }).done(function () {
                        target.empty();
                        self.pageModel.fetch();
                    });
                };

            if (this.options.app.models.user.get('deleteConfirmation')) {
                if (confirm('Are you sure you want to do delete the content?')) {
                    destroyModel();
                }
            } else {
                destroyModel();
            }
        }
    });

});
