/*global define*/
define([
    'sandbox/dom',
    'sandbox/view',
    'underscore'
], function ($, View, _) {

    'use strict';

    // This is a simple extension of
    // `Backbone.View`. The API works
    // as always with the exception that
    // one can provide a `contentDocumentEvents`
    // hash for delegating certain events
    // to `this.$el.get(0).contentDocument`.
    // So yeah, we kind of expect
    // `this.$el.is('iframe')`.

    var IFrame = View.extend({

        attributes: {
            'data-view': 'iframe'
        },

        // Let's perform a little dance to
        // take care of event delegation on both
        // the contentDocument and the iframe itself.
        // Of course, this is unit tested, such that
        // later updates of Backbone won't sweep us
        // off our feet.
        delegateEvents: function (events, contentDocumentEvents) {
            // First we will delegate `contentDocumentEvents`.
            var $contentDocument = $(this.$el.prop('contentDocument')),
                $originalEl = this.$el;

            contentDocumentEvents = contentDocumentEvents || _.result(this, 'contentDocumentEvents');

            // Announce that we are in the process of delegation.
            // See `undelegateEvents`.
            this._delegating = true;

            if (contentDocumentEvents && $contentDocument.length) {
                // For the time being, let's act like
                // `this.$el` is actually `$(this.$el).prop('contentDocument')`.
                this.$el = $contentDocument;
                IFrame.__super__.delegateEvents.call(this, contentDocumentEvents);
                // Reset `this.$el`.
                this.$el = $originalEl;
            }

            // Let's procede with the usual.
            IFrame.__super__.delegateEvents.call(this, events);

            // Announce that we are done with delegation.
            // See `undelegateEvents`.
            delete this._delegating;

            return this;
        },

        // The tricky part here is that
        // we do not want to undelegate
        // both the `events` and `contentDocumentEvents`
        // during delegation (see `delegateEvents`),
        // because `undelegateEvents` is called by
        // `__super__.delegateEvents` in both cases.
        // Hence the later delegation would undelegate
        // the first.
        undelegateEvents: function () {
            var $contentDocument, $originalEl;

            if (this._delegating) {
                return IFrame.__super__.undelegateEvents.apply(this, arguments);
            }
            else {
                $contentDocument = $(this.$el.prop('contentDocument'));
                $originalEl = this.$el;

                if ($contentDocument.length) {
                    // For the time being, let's act like
                    // `this.$el` is actually `$(this.$el).prop('contentDocument')`.
                    this.$el = $contentDocument;
                    IFrame.__super__.undelegateEvents.apply(this, arguments);
                    // Reset `this.$el`.
                    this.$el = $originalEl;
                }

                return IFrame.__super__.undelegateEvents.apply(this, arguments);
            }
        },

        // We need to make sure to
        // remove event handlers from
        // `contentDocument`.
        remove: function () {
            // First we need to undelegate
            // `contentDocumentEvents`
            // from `contentDocument`.
            var $contentDocument = $(this.$el.prop('contentDocument')),
                $originalEl = this.$el;

            if ($contentDocument.length) {
                this.$el = $contentDocument;
                IFrame.__super__.undelegateEvents.apply(this, arguments);
                this.$el = $originalEl;
            }

            // Now, on to the usual.
            return IFrame.__super__.undelegateEvents.apply(this, arguments);
        }
    });

    return IFrame;
});
