/*global define*/
define([
    'sandbox/dom',
    'sandbox/event',
    './iframe',
    'underscore'
], function ($, vent, Iframe, _) {

    'use strict';

    // This `Iframe` will be attached to the `iframe`
    // rendering the storefront.
    var Mouse = Iframe.extend({

        initialize : function () {
            // Remember what we're dragging.
            vent.on('drag:start', _.bind(this.setDragging, this));
        },

        // In order to have consistent behavior
        // across Chrome, Firefox and IE,
        // some events are attached directly to the
        // iframe, i.e. `this.$el`, while some others
        // **need** to be attached to
        // the iframe's `contentDocument`.
        events: {
            mouseleave: 'onLeave',
            mouseenter: 'onEnter',
            dragenter: 'onEnter'
        },

        contentDocumentEvents: {
            click: 'onClick',
            dragleave: 'onDragLeave',
            mousemove: 'onMove',
            dragover: 'onDragOver',
            drop: 'onDrop'
        },

        // We'll remember the instance of the *widget*
        // being dragged here.
        setDragging: function (widget) {
            this._dragging = widget;
        },

        clearHovered: function () {
            vent.trigger('hovered.last', null);
            this.lastHovered = null;
        },

        onClick: function (e) {
            vent.trigger('editor.content:change', e);
        },

        onMove : function (event, style) {
            if (this.lastHovered !== event.target) {
                this.lastHovered = event.target;
                vent.trigger('hovered.last', event.target, style, this.showActions(event.target));
            }
        },

        onDragOver : function (event) {
            var style = 'forbidden';

            // Let's ask `this._dragging`,
            // whether it would like to be dropped
            // right about now.
            if (this._dragging && _.isFunction(this._dragging.maybedropped)) {
                if (this._dragging.maybedropped(event)) {
                    style = 'droppable';
                }
            }

            // Do whatever needs to be done
            // on `mousemove`.
            this.onMove(event, style);

            // The `dragover` event needs to
            // be canceled in Google Chrome and Safari to
            // allow firing the `drop` event.
            if (style === 'droppable') {
                event.originalEvent.returnValue = false;
                event.preventDefault();
                return false;
            }
            return true;
        },

        // TODO
        // This will probably get more complicated over time.
        // For now, this is a point where `this` View,
        // decides whether to pass some additional options,
        // to the `hover` event.
        // Maybe at some point the elements will become
        // some javascript Model type thing, and these
        // things can be decided outside of the scope of
        // this `View`.
        showActions : function (element) {
            if ($(element).is('[data-type]')) {
                return true;
            } else {
                return false;
            }
        },

        onEnter : function () {
            // `mousemove` will be triggered next.
            // So let's clean up any leftovers.
            this.clearHovered();
        },

        onLeave : function (e) {
            // TODO
            // Not every `mouseleave` is the same :)
            // We want to keep the outline, if
            // the user is hovering over some UI
            // element contained by the outline.
            if (!$.contains(this.options.outline.$el.get(0), e.relatedTarget)) {
                this.clearHovered();
            }
        },

        onDragLeave : function () {
            this.clearHovered();
        },

        // Who knows what's supposed to happen, when
        // an element is dropped. Well, probably
        // the element which is being dragged!
        onDrop : function (event) {
            if (this._dragging) {
                this._dragging.onDrop(event);
                delete this._dragging;
            }
        }
    });

    return Mouse;
});
