/*global define, describe, beforeEach, it, expect, $, afterEach, setFixtures*/
define(['app/views/iframe'], function (Iframe) {

    'use strict';

    describe('View :: Iframe', function () {

        // Reference to the **Sytem Under Test**.
        var SUT, iframe, fixture;

        beforeEach(function () {
            fixture = setFixtures($('<iframe id="SUT"></iframe>'));
            iframe = fixture.find('iframe#SUT');
            SUT = new Iframe();
        });

        afterEach(function () {
            // Cleanup.
            SUT.remove();
            SUT = null;
        });


        describe('Event delegation', function () {

            describe('delegates `events` as usual', function () {
                var counter1, counter2, events;

                beforeEach(function () {
                    counter1 = 0;
                    counter2 = 0;
                    events = {click: 'increment'};

                    SUT.increment = function () {
                        counter2 = counter2 + 1;
                    };

                    delete SUT.contentDocumentEvents;
                    SUT.events = events;

                    SUT.setElement(iframe);

                    // For reference.
                    SUT.$el.on('click', function () {
                        counter1 = counter1 + 1;
                    });
                });

                it('to `this.$el`', function () {
                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(2);

                    SUT.delegateEvents(events);
                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(3);
                    expect(counter2).toEqual(3);
                });

                it('and undelegates these events', function () {
                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.undelegateEvents();

                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(1);
                });

                it('and undelegates these events on remove', function () {
                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.remove();

                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(1);
                });
            });

            describe('delegates `contentDocumentEvents`', function () {
                var counter1, counter2, contentDocumentEvents,
                    $contentDocument;

                beforeEach(function () {
                    counter1 = 0;
                    counter2 = 0;
                    contentDocumentEvents = {mousemove: 'increment'};

                    SUT.increment = function () {
                        counter2 = counter2 + 1;
                    };

                    SUT.contentDocumentEvents = contentDocumentEvents;
                    delete SUT.events;

                    SUT.setElement(iframe);

                    // For reference.
                    $contentDocument = $(iframe.prop('contentDocument'));
                    $contentDocument.on('mousemove', function () {
                        counter1 = counter1 + 1;
                    });
                });

                afterEach(function () {
                    $contentDocument.off('mousemove');
                });

                it('to `this.$el.prop(\'contentDocument\')`', function () {
                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(2);

                    SUT.delegateEvents({}, contentDocumentEvents);
                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(3);
                    expect(counter2).toEqual(3);
                });

                it('and undelegates contentDocumentEvents on remove', function () {
                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.remove();

                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(1);
                });

                it('and undelegates contentDocumentEvents', function () {
                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.undelegateEvents();

                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(1);
                });
            });
        });
    });
});
