/*global define, describe, beforeEach, spyOn, it, expect, $, afterEach, setFixtures, jasmine*/
define([
    'app/views/mouse', 'sandbox/event'
], function (Mouse, vent) {

    'use strict';

    describe('View :: Mouse', function () {

        // Reference to the **Sytem Under Test**.
        var SUT, iframe, fixture;

        beforeEach(function () {
            fixture = setFixtures($('<iframe id="SUT"></iframe>'));
            iframe = fixture.find('iframe#SUT');
            SUT = new Mouse();
        });

        afterEach(function () {
            // Cleanup.
            SUT.remove();
            SUT = null;
        });


        describe('Event delegation', function () {

            describe('delegates `events` as usual', function () {
                var counter1, counter2, events;

                beforeEach(function () {
                    counter1 = 0;
                    counter2 = 0;
                    events = {click: 'increment'};

                    SUT.increment = function () {
                        counter2 = counter2 + 1;
                    };

                    delete SUT.contentDocumentEvents;
                    SUT.events = events;

                    SUT.setElement(iframe);

                    // For reference.
                    SUT.$el.on('click', function () {
                        counter1 = counter1 + 1;
                    });
                });

                it('to `this.$el`', function () {
                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(2);

                    SUT.delegateEvents(events);
                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(3);
                    expect(counter2).toEqual(3);
                });

                it('and undelegates these events', function () {
                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.undelegateEvents();

                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(1);
                });

                it('and undelegates these events on remove', function () {
                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.remove();

                    SUT.$el.trigger('click');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(1);
                });
            });

            describe('delegates `contentDocumentEvents`', function () {
                var counter1, counter2, contentDocumentEvents,
                    $contentDocument;

                beforeEach(function () {
                    counter1 = 0;
                    counter2 = 0;
                    contentDocumentEvents = {mousemove: 'increment'};

                    SUT.increment = function () {
                        counter2 = counter2 + 1;
                    };

                    SUT.contentDocumentEvents = contentDocumentEvents;
                    delete SUT.events;

                    SUT.setElement(iframe);

                    // For reference.
                    $contentDocument = $(iframe.prop('contentDocument'));
                    $contentDocument.on('mousemove', function () {
                        counter1 = counter1 + 1;
                    });
                });

                afterEach(function () {
                    $contentDocument.off('mousemove');
                });

                it('to `this.$el.prop(\'contentDocument\')`', function () {
                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(2);

                    SUT.delegateEvents({}, contentDocumentEvents);
                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(3);
                    expect(counter2).toEqual(3);
                });

                it('and undelegates contentDocumentEvents on remove', function () {
                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.remove();

                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(1);
                });

                it('and undelegates contentDocumentEvents', function () {
                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(1);
                    expect(counter2).toEqual(1);

                    SUT.undelegateEvents();

                    $contentDocument.trigger('mousemove');
                    expect(counter1).toEqual(2);
                    expect(counter2).toEqual(1);
                });
            });
        });

        describe('Events and the `vent`', function () {
            var setLastHovered, $contentDocument;

            beforeEach(function () {
                // Set Element.
                SUT.setElement(iframe);
                $contentDocument = $(iframe.prop('contentDocument'));
                // Reset `lastHovered` for testing.
                setLastHovered = function (node) {
                    SUT.lastHovered = node;
                };

                // Init `spy` on `trigger.
                spyOn(vent, 'trigger').andCallThrough();
            });

            describe('On Enter', function () {
                beforeEach(function () {
                    iframe.trigger('mouseenter');
                });

                it('should trigger a null >hovered.last< event', function () {
                    expect(vent.trigger).toHaveBeenCalledWith('hovered.last', null);
                });
            });

            describe('On Move', function () {
                it('should trigger a null >hovered.last< event if the target is not yet hovered', function () {
                    $contentDocument.trigger('mousemove');
                    expect(vent.trigger).toHaveBeenCalledWith('hovered.last', $contentDocument.get(0), undefined, false);
                });


                it('shouldn\'t trigger any event if the target is already hovered', function () {
                    $contentDocument.trigger('mousemove');
                    $contentDocument.trigger('mousemove');
                    expect(vent.trigger.calls.length).toEqual(1);
                });

                // TODO
                // If you're failing this, you may possibly be improving the implementation.
                it('should show actions if the target element is data-type', function () {
                    var $dataType = $('<div data-type="text"></div>');
                    $contentDocument.find('body').append($dataType);
                    $dataType.trigger('mousemove');
                    expect(vent.trigger).toHaveBeenCalledWith('hovered.last', $dataType.get(0), undefined, true);
                });
            });

            describe('On Leave', function () {
                var $relatedTarget, $event;

                beforeEach(function () {
                    // Fake `outline`.
                    SUT.options.outline = {
                        $el: $('<div><div id="relatedtarget-inside-outline"></div></div>')
                    };
                });

                it('should trigger a null >hovered.last< event', function () {
                    iframe.trigger('mouseleave');
                    expect(vent.trigger).toHaveBeenCalledWith('hovered.last', null);
                });

                it('should **not** trigger a null >hovered.last< event, when triggered from within outline', function () {
                    $relatedTarget = SUT.options.outline.$el.find('#relatedtarget-inside-outline');
                    $event = $.Event('mouseleave', {
                        relatedTarget: $relatedTarget.get(0)
                    });
                    iframe.trigger($event);
                    expect(vent.trigger).not.toHaveBeenCalled();
                });
            });

            describe('On Drag Leave', function () {
                beforeEach(function () {
                    $contentDocument.trigger('dragleave');
                });

                it('should trigger a null >hovered.last< event', function () {
                    expect(vent.trigger).toHaveBeenCalledWith('hovered.last', null);
                });
            });


            // Check if ondrop of this.dragging was called!!!!
            describe('On Drop', function () {
                var contextOfOnDrop = null,
                    mockWidget = {
                        onDrop: function () {
                            contextOfOnDrop = this;
                        }
                    };

                beforeEach(function () {
                    contextOfOnDrop = null;
                    spyOn(mockWidget, 'onDrop').andCallThrough();
                });

                it('should call onDrop method of widget in context of the widget', function () {
                    // Start fake drag.
                    vent.trigger('drag:start', mockWidget);
                    // Drop it like its hot.
                    $contentDocument.trigger('drop');

                    expect(mockWidget.onDrop).toHaveBeenCalled();
                    expect(contextOfOnDrop).toBe(mockWidget);
                });
            });

            describe('On DragOver', function () {
                var contextOfMaybedropped = null,
                    $event,
                    mockWidget = {
                        maybedropped: function () {
                            contextOfMaybedropped = this;
                        }
                    };

                beforeEach(function () {
                    contextOfMaybedropped = null;
                    $event = $.Event('dragover', {
                        originalEvent: {}
                    });
                    spyOn(mockWidget, 'maybedropped').andCallThrough();
                    // Start fake drag.
                    vent.trigger('drag:start', mockWidget);
                });

                it('should ask the widget being dragged if it may be dropped', function () {
                    $contentDocument.trigger($event);
                    expect(mockWidget.maybedropped).toHaveBeenCalledWith(jasmine.any($.Event));
                    expect(contextOfMaybedropped).toBe(mockWidget);
                });

                describe('if the widget may be dropped', function () {
                    it('should trigger a last.hovered event with the correct style', function () {
                        mockWidget.maybedropped.andReturn(true);
                        $contentDocument.trigger($event);
                        expect(vent.trigger).toHaveBeenCalledWith('hovered.last', $event.target, 'droppable', false);
                    });

                    describe('or not', function () {
                        it('should trigger a last.hovered event with the correct style', function () {
                            mockWidget.maybedropped.andReturn(false);
                            $contentDocument.triggerHandler($event);
                            expect(vent.trigger).toHaveBeenCalledWith('hovered.last', $event.target, 'forbidden', false);
                        });
                    });

                });

            });
        });

    });
});
