/*global define, describe, beforeEach, afterEach, it, expect, sinon, jQuery, window*/
define(['app/views/pagelistitem', 'app/models/page', 'sandbox/event'], function (PageListItem, Page, vent) {

    'use strict';

    describe('views :: pagelistitem', function () {

        beforeEach(function () {
            this.app = {
                models: {
                    user: {
                        get: sinon.stub()
                    }
                }
            };
            this.view = new PageListItem({
                app: this.app
            });
            this.view.model = new Page();
            this.view.model.url = '/page/1';
        });

        afterEach(function () {
            this.view.close();
        });

        it('.initialize() should create a li dom element', function () {
            expect(this.view.el).toBe('li');
        });

        it('.render() should render a link with 3 span elements', function () {
            this.view.render();
            expect(this.view.el).toBe('li');
            expect(this.view.$el.find('a')).toHaveLength(1);
            expect(this.view.$el.find('a > span')).toHaveLength(4);
        });

        describe('eventing', function () {
            beforeEach(function () {
                this.spy = sinon.spy();
            });

            afterEach(function () {
                this.spy = null;
            });

            it('.openPage() should trigger a show event', function () {
                vent.on('editor.view:show', this.spy);
                this.view.openPage();
                expect(this.spy).toHaveBeenCalledOnce();
                expect(this.spy).toHaveBeenCalledWith(this.view.model);
            });

            it('.removePage() should trigger a close event', function () {
                vent.on('editor.view:close', this.spy);
                this.view.removePage();
                expect(this.spy).toHaveBeenCalledOnce();
                expect(this.spy).toHaveBeenCalledWith(this.view.model);
            });
        });

        describe('switchVisibility()', function () {
            beforeEach(function () {
                this.view.render();
                this.saveSpy = sinon.spy(this.view.model, 'save');
            });

            afterEach(function () {
                this.saveSpy.restore();
            });

            it('should save the model with visible true', function () {
                this.view.$el.find('.visibility-trigger').trigger('click');

                expect(this.saveSpy).toHaveBeenCalledWith('visible', true);
                expect(this.view.model.get('visible')).toBe(true);
            });

            it('should save the model with visible false', function () {
                this.view.model.set('visible', true);
                this.view.$el.find('.visibility-trigger').trigger('click');
                expect(this.saveSpy).toHaveBeenCalledWith('visible', false);
                expect(this.view.model.get('visible')).toBe(false);
            });

        });

        describe('.removePage()', function () {

            beforeEach(function () {
                this.confirmStub = sinon.stub(window, 'confirm').returns(true);

                this.deleteStub = sinon.stub(this.view.model, 'destroy');

                this.server = sinon.fakeServer.create();
                this.server.respondWith('DELETE', '/page/1', [204, {
                    'Content-Type': 'application/json'
                }, '']);
            });

            afterEach(function () {
                this.server.restore();
                this.confirmStub.restore();
                this.deleteStub.restore();
            });

            it('must delete the page', function () {
                this.app.models.user.get.returns(false);
                this.view.removePage();
                this.server.respond();

                expect(this.deleteStub).toHaveBeenCalledOnce();
                expect(this.confirmStub).not.toHaveBeenCalled();
            });

            it('must delete the page with confirm', function () {
                this.app.models.user.get.returns(true);
                this.view.removePage();
                this.server.respond();

                expect(this.deleteStub).toHaveBeenCalledOnce();
                expect(this.confirmStub).toHaveBeenCalled();
            });

            it('must *not* delete the page with confirm', function () {
                this.app.models.user.get.returns(true);
                this.confirmStub.returns(false);
                this.view.removePage();
                this.server.respond();

                expect(this.deleteStub).not.toHaveBeenCalled();
                expect(this.confirmStub).toHaveBeenCalled();
            });

        });

        describe('.changeName()', function () {
            beforeEach(function () {
                this.view.model.saved = false;
                this.event = jQuery.Event('keypress');
                this.view.model.save = function () { this.saved = true; };
                this.view.render();
                this.view.changeName();
            });

            it('entering a character does not save the page', function () {
                this.event.which = 99;

                this.view.$('input').trigger(this.event);

                expect(this.view.model.saved).toBe(false);
            });

            it('changing the page name and pressing enter saves the page', function () {
                this.view.$el.find('#page-name').val('NewPageName');
                this.event.which = 13;

                this.view.$('input').trigger(this.event);

                expect(this.view.model.get('name')).toBe('NewPageName');
                expect(this.view.model.saved).toBe(true);
            });

            it('not changing the page name and pressing enter does not save the page', function () {
                this.view.$el.find('#page-name').val(this.view.model.get('name'));
                this.event.which = 13;

                this.view.$('input').trigger(this.event);

                expect(this.view.model.saved).toBe(false);
            });
        });

    });
});

